@@ -161,7 +161,19 @@ sub process_limits {
     my $work_array = [];
     foreach my $workfield (
         @{munin_find_field_for_limits($config, qr/^(critical|warning)/)}) {
-        my $parent = munin_get_parent($workfield);
+        my $parent;
+        if (defined $workfield->{'graph_title'}) {
+            # Limit is defined on a service and inherits to all fields, queue it
+            $parent = $workfield;
+        } else {
+            # Limit is defined on a field, or a non-existent service
+            # Assume field, grab parent service, and verify it is valid
+            $parent = munin_get_parent($workfield);
+            if (!defined $parent->{'graph_title'}) {
+                DEBUG "[DEBUG] Ignoring work item for non-existent service: " . munin_get_node_name($parent) if ($DEBUG);
+                next;
+            }
+        }
         if (!defined $work_hash_tmp{$parent}) {
 	    $work_hash_tmp{$parent} = 1;
 	    push @$work_array, $parent;
@@ -297,7 +309,7 @@ sub process_service {
     my $hparentobj = munin_get_parent($hobj);
     my $parent     = munin_get_node_name($hobj);
     my $gparent    = munin_get_node_name($hparentobj);
-    my $children   = munin_get_children($hash);
+    my $field_order = munin_get_field_order($hash);
 
     if (!ref $hash) {
 	LOGCROAK("I was passed a non-hash!");
@@ -308,72 +320,75 @@ sub process_service {
     DEBUG "[DEBUG] processing service: $service";
 
     # Some fields that are nice to have in the plugin output
-    $hash->{'fields'} = join(' ', map {munin_get_node_name($_)} @$children);
+    $hash->{'fields'} = join(' ', @$field_order);
     $hash->{'plugin'} = $service;
     $hash->{'graph_title'} = get_full_service_name($hash);
     $hash->{'host'}  = $hostalias;
     $hash->{'group'} = get_full_group_path($hparentobj);
     $hash->{'worst'} = "ok";
     $hash->{'worstid'} = 0 unless defined $hash->{'worstid'};
+    $hash->{'recovered'} = {};
 
     my $state_file = sprintf ('%s/state-%s-%s.storable', $config->{dbdir}, $hash->{group}, $host);
     DEBUG "[DEBUG] state_file: $state_file";
     my $state = munin_read_storable($state_file) || {};
 
-    foreach my $field (@$children) {
+    my %seen = ();
+    foreach my $fname (@$field_order) {
+        # If field has an alias, strip it away and store it for use in munin_get_rrd_filename
+        my $path = undef;
+        $path = $1 if ($fname =~ s/=(.+)//);
+
+        # Field order contains duplicates sometimes, skip if already seen
+        next if (exists($seen{$fname}));
+        $seen{$fname} = 1;
+
+        my $field   = munin_get_node($hash, [$fname]);
         next if (!defined $field or ref($field) ne "HASH");
-        my $fname   = munin_get_node_name($field);
         my $fpath   = munin_get_node_loc($field);
         my $onfield = munin_get_node($oldnotes, $fpath);
-	my $oldstate= '';
-
-	# Test directly here as get_limits is in truth recursive and
-	# that fools us when processing multigraphs.
-	next if (!defined($field->{warning}) and !defined($field->{critical}));
-
-	# get the old state if there is one, or leave it empty.
-	if ( defined($onfield) or
-	     defined($onfield->{"state"}) ) {
-	    $oldstate = $onfield->{"state"};
-	}
+        my $oldstate = 'ok';
 
         my ($warn, $crit, $unknown_limit) = get_limits($field);
 
         # Skip fields without warning/critical definitions
         next if (!defined $warn and !defined $crit);
 
+	# get the old state if there is one, or leave it empty.
+	if ( defined($onfield) and
+	     defined($onfield->{"state"}) ) {
+	    $oldstate = $onfield->{"state"};
+	}
+
         DEBUG "[DEBUG] processing field: " . join('::', @$fpath);
         DEBUG "[DEBUG] field: " . munin_dumpconfig_as_str($field);
 	my $value;
     	{
-		my $rrd_filename = munin_get_rrd_filename($field);
+		my $rrd_filename = munin_get_rrd_filename($field, $path);
 		my ($current_updated_timestamp, $current_updated_value) = @{ $state->{value}{"$rrd_filename:42"}{current} || [ ] };
 		my ($previous_updated_timestamp, $previous_updated_value) = @{ $state->{value}{"$rrd_filename:42"}{previous} || [ ] };
 
 		my $heartbeat = 600; # XXX - $heartbeat is a fixed 10 min (2 runs of 5 min).
-		if (! $field->{type} || $field->{type} eq "GAUGE") {
-			$value = $current_updated_value;
-		} elsif (! defined $current_updated_value || ! defined $previous_updated_value || $current_updated_timestamp == $previous_updated_timestamp) {
-			# No derive computing possible. Report unknown.
+		if (! defined $current_updated_value || $current_updated_value eq "U") {
+			# No value yet. Report unknown.
 			$value = "U";
 		} elsif (time > $current_updated_timestamp + $heartbeat) {
-			# Current value is too old. Report unknown. 
+			# Current value is too old. Report unknown.
 			$value = "U";
-		} elsif ($current_updated_timestamp > $previous_updated_timestamp + $heartbeat) {
-			# Old value is too old. Report unknown. 
+		} elsif (! $field->{type} || $field->{type} eq "GAUGE") {
+			# Non-compute up-to-date value.
+			$value = $current_updated_value;
+		} elsif (! defined $previous_updated_value || $previous_updated_value eq "U") {
+			# No derive computing possible. Report unknown.
 			$value = "U";
-		} elsif ($field->{type} eq "COUNTER" && $current_updated_value < $previous_updated_value) {
-			# COUNTER never decrease. Report unknown.
-			$value = "U";
-		} elsif ($current_updated_value eq "U") {
-			# The current value is unknown. Report unknown.
+		} elsif ($current_updated_timestamp == $previous_updated_timestamp || $current_updated_timestamp > $previous_updated_timestamp + $heartbeat) {
+			# Old value does not exist or is too old. Report unknown.
 			$value = "U";
 		} elsif ($field->{type} eq "ABSOLUTE") {
-			# The previous value is unimportant, as if ABSOLUTE, the counter is reset anytime the value is read
+			# The previous value is unimportant, as if ABSOLUTE, the counter is reset everytime the value is read
 			$value = $current_updated_value / ($current_updated_timestamp - $previous_updated_timestamp);
-		} elsif ($previous_updated_value eq "U") {
-			# The previous value is unknown.
-			# Report unknown, as we are not ABSOLUTE
+		} elsif ($field->{type} eq "COUNTER" && $current_updated_value < $previous_updated_value) {
+			# COUNTER never decrease. Report unknown.
 			$value = "U";
 		} else {
 			# Everything is ok for DERIVE/COUNTER
@@ -383,7 +398,7 @@ sub process_service {
 	}
 
         # De-taint.
-        if (!defined $value || $value eq "U") {
+        if ($value eq "U") {
             $value = "unknown";
         }
         else {
@@ -525,6 +540,7 @@ sub process_service {
 
 	    if ($oldstate ne 'ok') {
 		$hash->{'state_changed'} = 1;
+		$hash->{'recovered'}{$fname} = 1;
 	    }
         }
     }
@@ -536,8 +552,8 @@ sub get_limits {
     my $hash = shift || return;
 
     # This hash will have values that we can look up such as these:
-    my @critical = (undef, undef);
-    my @warning  = (undef, undef);
+    my $critical = undef;
+    my $warning  = undef;
     my $crit          = munin_get($hash, "critical",      undef);
     my $warn          = munin_get($hash, "warning",       undef);
     my $unknown_limit = munin_get($hash, "unknown_limit", 3);
@@ -545,37 +561,38 @@ sub get_limits {
     my $name = munin_get_node_name($hash);
 
     if (defined $crit and $crit =~ /^\s*([-+\d.]*):([-+\d.]*)\s*$/) {
-        $critical[0] = $1 if length $1;
-        $critical[1] = $2 if length $2;
+        $critical = [undef, undef];
+        ${$critical}[0] = $1 if length $1;
+        ${$critical}[1] = $2 if length $2;
     }
     elsif (defined $crit and $crit =~ /^\s*([-+\d.]+)\s*$/) {
-        $critical[1] = $1;
+        $critical = [undef, $1];
     }
     elsif (defined $crit) {
-        @critical = (0, 0);
+        $critical = [0, 0];
     }
     if(defined $crit) {
         DEBUG "[DEBUG] processing critical: $name -> "
-                . (defined $critical[0]? $critical[0] : "")
+                . (defined ${$critical}[0]? ${$critical}[0] : "")
                 .  " : "
-                . (defined $critical[1]? $critical[1] : "");
-    }   
+                . (defined ${$critical}[1]? ${$critical}[1] : "");
+    }
 
     if (defined $warn and $warn =~ /^\s*([-+\d.]*):([-+\d.]*)\s*$/) {
-        $warning[0] = $1 if length $1;
-        $warning[1] = $2 if length $2;
+        ${$warning}[0] = $1 if length $1;
+        ${$warning}[1] = $2 if length $2;
     }
     elsif (defined $warn and $warn =~ /^\s*([-+\d.]+)\s*$/) {
-        $warning[1] = $1;
+        $warning = [undef, $1];
     }
     elsif (defined $warn) {
-        @warning = (0, 0);
+        $warning = [0, 0];
     }
     if(defined $warn) {
         DEBUG "[DEBUG] processing warning: $name -> "
-                . (defined $warning[0]? $warning[0] : "")
+                . (defined ${$warning}[0]? ${$warning}[0] : "")
                 .  " : "
-                . (defined $warning[1]? $warning[1] : "");
+                . (defined ${$warning}[1]? ${$warning}[1] : "");
     }
 
     if ($unknown_limit =~ /^\s*(\d+)\s*$/) {
@@ -589,7 +606,7 @@ sub get_limits {
         DEBUG "[DEBUG] processing unknown_limit: $name -> $unknown_limit";
     }
 
-    return (\@warning, \@critical, $unknown_limit);
+    return ($warning, $critical, $unknown_limit);
 }
 
 sub generate_service_message {
@@ -616,10 +633,11 @@ sub generate_service_message {
     if ( defined($children) ) {
 	foreach my $field (@$children) {
 	    if (defined $field->{"state"}) {
-		push @{$stats{$field->{"state"}}}, munin_get_node_name($field);
-		if ($field->{"state"} eq "ok") {
-		    push @{$stats{"foks"}}, munin_get_node_name($field);
-		}
+                my $fname = munin_get_node_name($field);
+                push @{$stats{$field->{'state'}}}, $fname;
+                if ($field->{'state'} eq 'ok' and defined $hash->{'recovered'}{$fname}) {
+                    push @{$stats{'foks'}}, $fname;
+                }
 	    }
 	}
     }
@@ -674,7 +692,7 @@ sub generate_service_message {
         if (!$hash->{'state_changed'} and !$obsess) {
             next;    # No need to send notification
         }
-        DEBUG "[DEBUG] state has changed, notifying $c";
+        INFO("[INFO] state has changed, notifying $c");
         my $precmd = munin_get($contactobj, "command", undef);
         if(!defined $precmd) {
             WARN("[WARNING] Missing command option for contact $c; skipping");
@@ -731,6 +749,12 @@ sub generate_service_message {
             } else { # child
                 close $w;
                 open(STDIN, "<&", $r);
+                # We want to close stdout before calling the send command. This prevents the
+                # notification program (e.g. "send_nsca") to write irrelevant status messages
+                # to stdout and thus trigger notification emails (e.g. via cron).
+                # See https://github.com/munin-monitoring/munin/issues/382
+                # and https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=291168.
+                close(STDOUT);
                 exec($cmd) or WARN "[WARNING] Failed to exec for contact $c in pid $$";
                 exit;
             }
@@ -839,7 +863,7 @@ will return a reference to an empty array.
 
 sub validate_severities {
     my $severities_ref = shift;
-    my @severities     = @{$severities_ref};
+    my @severities     = grep { defined $_ }  @{$severities_ref};
 
     my @allowed_severities = qw{ok warning critical unknown};
 
