@@ -475,10 +475,10 @@ sub get_version {
 
     return if (defined $self->{detected_version});
 
-    my $r = $self->runquery("SELECT version()");
+    my $r = $self->runquery("SHOW server_version");
     my $v = $r->[0]->[0];
     die "Unable to detect PostgreSQL version\n"
-        unless ($v =~ /^PostgreSQL (\d+)\.(\d+)(\.\d+|devel|beta\d+)\b/);
+        unless ($v =~ /^(\d+)\.(\d+).*\b/);
     $self->{detected_version} = "$1.$2";
 }
 
